#!/usr/bin/bash

# links
nginxversion=`nginx -v |& sed 's/nginx version: nginx\///'`
nginxdlurl="https://nginx.org/download/nginx-$nginxversion.tar.gz"
nginxmodsecurl="https://github.com/SpiderLabs/ModSecurity-nginx"
nginxgeoipurl="https://github.com/leev/ngx_http_geoip2_module"

# vars
checknginxurl=`curl -s -I $nginxdlurl | grep HTTP | awk {'print $2'}`
checkmodsec=`curl -s -I $nginxmodsecurl | grep HTTP | awk {'print $2'}`
checkgeoipurl=`curl -s -I $nginxgeoipurl | grep HTTP | awk {'print $2'}`

#nginxmodulepath=`nginx -V 2>&1 | tr ' ' '\n' | grep module | grep modules-path | sed -e 's/--modules-path=//g'`
nginxmodulepath="/etc/nginx/modules/"
downloaddir="/root/nginxbuild"

# check if nginx download link is valid
if [ "$checknginxurl" != "200" ]; then
	echo "No valid nginx download link. Please check -> $nginxdlurl"
	exit;
fi

# check if nginx extras git link is valid
if [ "$checkmodsec" != "200" ]; then
        echo "No valid nginx mod_security git link. Please check -> $checkmodsec"
        exit;
fi

# check and create download dir
if [ ! -d  "$downloaddir" ]; then
        mkdir $downloaddir;
fi

# install development tools and dependecyes
dnf group install "Development Tools" -y
dnf install GeoIP GeoIP-devel pcre pcre-devel libmaxminddb libmaxminddb-devel


# download nginx binary
wget $nginxdlurl -P $downloaddir
git clone "$nginxmodsecurl.git" "$downloaddir/ModSecurity-nginx"
git clone "$nginxgeoipurl.git" "$downloaddir/ngx_http_geoip2_module"
git clone "$nginxmodsecurl.git" "$downloaddir/ModSecurity-nginx"
git clone --depth 1 -b v3/master --single-branch https://github.com/SpiderLabs/ModSecurity "$downloaddir/ModSecurity"

# install modsecurity
cd "$downloaddir/ModSecurity" && git submodule init; git submodule update; ./build.sh; ./configure --with-maxmind=no; make; make install

# unarchive nginx
tar -xvzf "$downloaddir/nginx-$nginxversion.tar.gz" -C $downloaddir

# configure and build modules
cd "$downloaddir/nginx-$nginxversion/" && ./configure --add-dynamic-module="../ngx_http_geoip2_module"  --add-dynamic-module="../ModSecurity-nginx" --with-compat && make modules

if [ ! -f "$downloaddir/nginx-$nginxversion/objs/ngx_http_geoip2_module.so" ]; then
        echo "No GeoIP module generated."
        exit;
fi

if [ ! -f "$downloaddir/nginx-$nginxversion/objs/ngx_http_modsecurity_module.so" ]; then
        echo "No mod_security module generated."
        exit;
fi

# move generated modules to modules directory
mv "$downloaddir/nginx-$nginxversion/objs/ngx_http_geoip2_module.so" "$nginxmodulepath"
mv "$downloaddir/nginx-$nginxversion/objs/ngx_http_modsecurity_module.so" "$nginxmodulepath"

cp "$downloaddir/ModSecurity/modsecurity.conf-recommended" /etc/nginx/conf.d/modsecurity.conf
cp "$downloaddir/ModSecurity/unicode.mapping" /etc/nginx/conf.d/

# download and setup owasp rulesets
cd /etc/nginx && git clone https://github.com/SpiderLabs/owasp-modsecurity-crs.git
cp /etc/nginx/owasp-modsecurity-crs/crs-setup.conf.example /etc/nginx/owasp-modsecurity-crs/crs-setup.conf

# inject module path to nginx config

sed -i '1i \
# aditional modules \
load_module modules/ngx_http_geoip2_module.so; \
load_module modules/ngx_http_modsecurity_module.so; \
' /etc/nginx/nginx.conf

nginx -t &>/dev/null
if [ "$?" != "0" ]; then
        echo "Error in Nginx configuration. Please check. Skip reloading ...";
	exit;
else
        echo "Everything fine, reloading Nginx.";
	systemctl reload nginx
	rm -rf $downloaddir
fi